# Copyright 2025 The MelowRAG Author @erfanzar (Erfan Zare Chavoshi).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re

from .base import Task


class FileTask(Task):
    """
    Task that processes file paths
    """

    FILE = r"file:\/\/"

    def accept(self, element):
        element = re.sub(FileTask.FILE, "", element)

        return super().accept(element) and isinstance(element, str) and os.path.exists(element)

    def prepare(self, element):
        return re.sub(FileTask.FILE, "", element)
